﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Windows;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.WpfDesktopEdition;

namespace DisplayASimpleMap
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        //private int pointsCount = 100000;
        private int pointsCount = 10000;
        private static Random random = new Random();
        public MainWindow()
        {
            InitializeComponent();
        }

        private void WpfMap_Loaded(object sender, RoutedEventArgs e)
        {
            wpfMap1.MapUnit = GeographyUnit.DecimalDegree;
            wpfMap1.CurrentExtent = new RectangleShape(-163.4, 41.9, -60.1, 30.3);
            wpfMap1.BackgroundOverlay.BackgroundBrush = new GeoSolidBrush(GeoColor.GeographicColors.ShallowOcean);

            InMemoryFeatureLayer pointsLayer = new InMemoryFeatureLayer();
            //pointsLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle = PointStyles.City1;
            pointsLayer.ZoomLevelSet.ZoomLevel01.CustomStyles.Add(new CustomWellSymbolStyle());
            pointsLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            LayerOverlay pointsOverlay = new LayerOverlay();
            pointsOverlay.TileType = TileType.SingleTile;
            pointsOverlay.Layers.Add("PointsLayer", pointsLayer);
            wpfMap1.Overlays.Add("PointsOverlay", pointsOverlay);

            wpfMap1.Refresh();
        }

        private void btnRefresh_Click(object sender, RoutedEventArgs e)
        {
            LayerOverlay pointsOverlay = (LayerOverlay)wpfMap1.Overlays["PointsOverlay"];
            InMemoryFeatureLayer pointsLayer = (InMemoryFeatureLayer)pointsOverlay.Layers["PointsLayer"];

            pointsLayer.InternalFeatures.Clear();
            for (int i = 0; i < pointsCount; i++)
            {
                PointShape pointShape = GetRandomPoint(wpfMap1.CurrentExtent);
                Feature feature = new Feature(pointShape.X, pointShape.Y);
                pointsLayer.InternalFeatures.Add(feature);
            }

            pointsOverlay.Refresh();
        }

        private PointShape GetRandomPoint(RectangleShape currentExtent)
        {
            double x = random.Next((int)currentExtent.UpperLeftPoint.X, (int)currentExtent.LowerRightPoint.X);
            double y = random.Next((int)currentExtent.LowerRightPoint.Y, (int)currentExtent.UpperLeftPoint.Y);

            PointShape pointShape = new PointShape(x, y);
            return pointShape;
        }
    }

    public class CustomWellSymbolStyle : PointStyle
    {
        protected override void DrawCore(IEnumerable<Feature> features, GeoCanvas canvas, System.Collections.ObjectModel.Collection<SimpleCandidate> labelsInThisLayer, System.Collections.ObjectModel.Collection<SimpleCandidate> labelsInAllLayers)
        {
            GeoFont characterFont = new GeoFont("Arial", 8);
            GeoSolidBrush geoBrush = new GeoSolidBrush(GeoColor.StandardColors.Navy);
            GeoPen geoPen = new GeoPen(GeoColor.StandardColors.Black);

            List<ScreenPointF> allScreenPoints = new List<ScreenPointF>();
            int index = 1;
            Stopwatch watch = new Stopwatch();
            watch.Start();
            foreach (Feature feature in features)
            {
                RectangleShape worldExtent = canvas.CurrentWorldExtent;
                ScreenPointF screenPoint = ExtentHelper.ToScreenCoordinate(worldExtent, feature, canvas.Width,
                                                                  canvas.Height);
                //bool uniquePoint = !allScreenPoints.Contains(screenPoint);

                //if (uniquePoint)
                {
                    allScreenPoints.Add(screenPoint);
                    ScreenPointF[] screenPoints = new[] { new ScreenPointF(screenPoint.X, screenPoint.Y) };
                    canvas.DrawText(string.Format("{0}", (char)index), characterFont, geoBrush, geoPen, screenPoints,
                                    DrawingLevel.LabelLevel, 0, 0, 0);
                }
                index++;
            }
            watch.Stop();
            Console.Write("time"+watch.ElapsedMilliseconds);
        }
    }
}
